@@ -102,6 +102,7 @@ uint InputCapturePortal::CreateSession(const QDBusObjectPath &handle,
     InputCaptureDialog dialog(app_id, Capabilities::fromInt(requestedCapabilities), this);
     Utils::setParentWindow(dialog.windowHandle(), parent_window);
     Request::makeClosableDialogRequest(handle, &dialog);
+    connect(session, &Session::closed, &dialog, &QuickDialog::reject);
 
     if (!dialog.exec()) {
         return 1;
@@ -207,7 +208,7 @@ uint InputCapturePortal::GetZones(const QDBusObjectPath &handle,
             .x_offset = screen->geometry().x(),
             .y_offset = screen->geometry().y(),
         });
-        connect(screen, &QScreen::geometryChanged, this, handleZoneChange);
+        connect(screen, &QScreen::geometryChanged, session, handleZoneChange);
     }
 
     connect(qGuiApp, &QGuiApplication::screenAdded, session, handleZoneChange);
