@@ -24,8 +24,54 @@
 #include <QRegion>
 #include <QScreen>
 
+#include "permission_store.h"
+
 using namespace Qt::StringLiterals;
 
+namespace
+{
+
+// This is a helper function to check if the application is mega-authorized.
+// Mega-authorization is a permission system specific to the KDE portal implementation.
+// A mega-authorized application is one that has been granted permissions to access all (remote desktop) features **without** any further user interaction.
+// This function should be used to check if a UI interaction can be skipped right away.
+// For instance if an application is mega-authorized we don't need to ask the user if they want to allow keyboard/mouse input. It's always authorized.
+// Particularly useful for headless setups and when the user is not physically at the machine.
+bool isAppMegaAuthorized(const QString &app_id)
+{
+    // NOTE: an empty app_id should never occur for flatpak/snap applications and as such is meant to denote a host application
+    //   of which the app_id is not known. In such a case the user may authorize the empty app_id to cover generic host applications.
+    //   Specifically xwayland may request input permissions but has no app_id.
+    //   Note that this is different from giving out an "any" permission. An application that has an app_id will not be covered by the empty rule.
+    qDBusRegisterMetaType<AppIdPermissionsMap>();
+    OrgFreedesktopImplPortalPermissionStoreInterface permissionStore(u"org.freedesktop.impl.portal.PermissionStore"_s,
+                                                                     u"/org/freedesktop/impl/portal/PermissionStore"_s,
+                                                                     QDBusConnection::sessionBus());
+    // Bring the timeout way down. Permission store queries are fast, if they aren't then something is wrong and there is no point waiting a long time.
+    permissionStore.setTimeout(1000);
+    QDBusVariant data;
+    auto reply = permissionStore.Lookup(u"kde-authorized"_s, u"remote-desktop"_s, data);
+    if (reply.isValid()) {
+        auto appIdPermissions = reply.value();
+        if (!appIdPermissions.contains(app_id)) {
+            qCDebug(XdgDesktopPortalKdeRemoteDesktop) << "MegaAuth: Permission not granted for" << app_id;
+            return false;
+        }
+
+        auto permissions = appIdPermissions.value(app_id);
+        if (permissions.contains("yes"_L1)) {
+            qCDebug(XdgDesktopPortalKdeRemoteDesktop) << "MegaAuth: Permission granted for" << app_id;
+            return true;
+        }
+    } else {
+        qCWarning(XdgDesktopPortalKdeRemoteDesktop) << "MegaAuth: Failed to lookup permissions:" << reply.error().message();
+    }
+
+    return false;
+};
+
+} // namespace
+
 static QString kwinService()
 {
     return QStringLiteral("org.kde.KWin");
@@ -178,14 +224,16 @@ uint RemoteDesktopPortal::Start(const QDBusObjectPath &handle,
         notification->setIconName(QStringLiteral("krfb"));
         notification->sendEvent();
     } else {
-        QScopedPointer<RemoteDesktopDialog, QScopedPointerDeleteLater> remoteDesktopDialog(
-            new RemoteDesktopDialog(app_id, session->deviceTypes(), session->screenSharingEnabled(), session->persistMode()));
-        Utils::setParentWindow(remoteDesktopDialog->windowHandle(), parent_window);
-        Request::makeClosableDialogRequest(handle, remoteDesktopDialog.get());
-        connect(session, &Session::closed, remoteDesktopDialog.data(), &RemoteDesktopDialog::reject);
+        if (!isAppMegaAuthorized(app_id)) { // authorize right away
+            QScopedPointer<RemoteDesktopDialog, QScopedPointerDeleteLater> remoteDesktopDialog(
+                new RemoteDesktopDialog(app_id, session->deviceTypes(), session->screenSharingEnabled(), session->persistMode()));
+            Utils::setParentWindow(remoteDesktopDialog->windowHandle(), parent_window);
+            Request::makeClosableDialogRequest(handle, remoteDesktopDialog.get());
+            connect(session, &Session::closed, remoteDesktopDialog.data(), &RemoteDesktopDialog::reject);
 
-        if (!remoteDesktopDialog->exec()) {
-            return 1;
+            if (!remoteDesktopDialog->exec()) {
+                return 1;
+            }
         }
     }
 
